#!/usr/bin/perl

use strict;
use warnings;
use lib "./";   # add current Dir to @inc

use MPEGSTREAM;

my $debug          = 0;
my $path_param     = "./";
my @parameter_list = @ARGV;
my $tcmplex        = "";
my $mplex          = 0;
my $transcode      = 0;
my $info           = 0;
my $remuxfilename  = ""; 
my @ignore_streams;
my $dump_packets   = 0;
my $dump_payload   = 0;
my $stop_flag      = 0;
my $divx   	   = 0;
my $divx_param     = " -y divx4 -w 1000 "; #enter additional parameters for divx here

if (-e "/usr/bin/tcmplex")
	{
	$tcmplex = "/usr/bin/tcmplex";
	}
elsif (-e "/usr/local/bin/tcmplex")
	{
	$tcmplex = "/usr/local/bin/tcmplex";
	}
if (-e "/usr/bin/transcode")
	 {
	$transcode = "/usr/bin/transcode";
	}
	elsif (-e "/usr/local/bin/transcode")
	{
	$transcode = "/usr/local/bin/transcode";
	}
 

sub parse_parameters
	{
	my $parameter = shift;
	if ($parameter eq "-d")
		{
		$debug = 1;
		}
	elsif ($parameter eq "-o")
		{
		$path_param = shift @parameter_list;
		}
	elsif ($parameter eq "-m")
		{
		$mplex = 1;
		}
	elsif ($parameter eq "-i")
		{
		$info = 1;
		}
	elsif ($parameter eq "-ignore")
		{
		 @ignore_streams = split /,/,(shift @parameter_list);
		}
	elsif ($parameter eq "-dump-packets")
		{
		$dump_packets = (shift @parameter_list);
		}
	elsif ($parameter eq "-dump-payload")
		{
		$dump_payload = (shift @parameter_list);
		}
	elsif ($parameter eq "-stop")
		{
		$stop_flag = 1;
		}
	elsif ($parameter eq "-divx")
		{
		$divx = 1;
		}	
	}

my @files;
if (scalar(@ARGV) == 0) 
	{
	print "\nUsage:   vdrsync.pl vdrfile1 .... vdrfilen\n\n";
	print "Optional: -d print strange messages and frequently stops the script\n";
	print "           until you press enter\n";
	print "Optional: -o output_dir sets the directory where the result files\n";
	print "           are written to\n";
	print "Optional: -m tries to multiplex the demuxed streams into a dvd compatible\n";
	print "           stream (uses tcmplex)\n";
	print "Optional: -i just tries to analyse the .vdr file and exits. -m is ignored\n";
	print "Optional: -ingore stream1[,stream2]...  tells the script to ignore  stream1..n \n"; 
	print "           Useful for deliberately omitting streams (like AC3) or if you have \n";
	print "           trouble with one stream\n";
	print "Optional: -divx tries to transcode the demuxed streams into a divx movie\n";
        print "           (uses transcode, adjust transcode settings at the beginning of\n";
	print "           vdrsync.pl)\n";
	print "Optional: -dump-packet NNNN tells the script to dump  the first NNNN\n";
	print "           full PES-packets (in file STREAM_ID.pes_dump)Useful for debugging \n"; 
	print "           (if you want to extract a stream that does not work and mail it to me)\n";
	print "Optional: -dump-payload NNNN tells the script to dump the raw payload of the \n";
	print "           first NNNN PES-packets (in file STREAM_ID.dump)Useful for debugging \n"; 
	print "           (if you want to extract a stream-payload that does not work and \n";
	print "           test it with other tools)\n";
	print "Optional  -stop stops the script frequently to allow you to read the debug output\n";
	print "           probably only used by myself ;) \n";
	exit
	} 


while (my $param = shift(@parameter_list))  # check whether all files hat we got as parameters exist
	{
	print "Got parameter $param\n";
	if ($param =~ /^-/) {parse_parameters($param); next}
	push @files, $param;
	}

print "Output files will be stored in $path_param\n" if $debug;

if (!(-e $path_param)) {print "Output Directory does $path_param not exist\n"; exit}



print "Found " . (scalar(@ignore_streams)) ." to ignore\n";

my %ignore_hash;
foreach (@ignore_streams)
	{
	$ignore_hash{$_} = 1;
	}

foreach (keys(%ignore_hash))
	{
	print "$_ found in ignorelist\n";
	}


print "dump_payload is $dump_payload\n";

my $PES_Obj = MPEGSTREAM->new
		(
		streamcode	=>"PES_stream", 
		debug		=>$debug, 
		outputpath	=>$path_param, 
		files		=>\@files, 
		ignore_streams 	=> \%ignore_hash,
		dump_packets	=> $dump_packets,
		dump_payload	=> $dump_payload,
		stop		=> $stop_flag,
		);

if (!($PES_Obj))
	{
	print "we could not create the PES Stream Obj\n";
	}
exit if $info;


$PES_Obj->process_PES_packets();


if ((!$mplex == 1) && (!$divx == 1)) {exit}

my @filestomux = @{$PES_Obj->output_files()};

if (scalar(@filestomux) < 2) {print "Need at least two streams for multiplexing\n";exit}

my @basename_list;
foreach (@filestomux)
	{
	my $base_name = (split /\//,$_)[-1];
	
	$base_name =~ s/\.//;
	
	push @basename_list, $base_name;
	}

if ($divx == 1) 
	{
 	my @filestomux = @{$PES_Obj->output_files()};
	if (scalar(@filestomux) < 2) {print "Need at least two streams for multiplexing\n";exit}
 
	#transcode -i e0.mpv -p c0.mpa -y divx4 -o test.avi
	my $command = "$transcode -i $filestomux[0] -p $filestomux[1]  ";
	$remuxfilename  .= $path_param . "/$basename_list[0]_$basename_list[1]_";
 	$command .= " $divx_param -o $remuxfilename" . "divx.avi";
	print "executeing $command\n";
 	system $command;
 	exit;
	}

my $command = "$tcmplex -i $filestomux[0] -p $filestomux[1]";
$remuxfilename  .= $path_param . "/$basename_list[0]_$basename_list[1]_";

if (scalar(@filestomux) > 2)
	{
	$command .= " -s $filestomux[2]";
	$remuxfilename .= "$basename_list[2]_";
	}
$command .= " -m d -o $remuxfilename" . "remux.mpg";

print "executeing $command\n";

system $command;

exit;

